# coding: utf-8
require 'open3'
require 'java_submission_tester/java_helper'

module JavaSubmissionTester
  class JavaSubmissionTestRunner
    include ::JavaSubmissionTester::JavaHelper

    LANGUAGE_NAME = 'Java'

    def validate_test_structure(test_file, stub_file)
      validation_message = validate_is_jar? test_file
      validation_message = validate_is_jar? stub_file

      return validation_message unless validation_message.nil?

      run_unzipped test_file do |path|
        pathname = Pathname.new path

        validation_message = 'Test muss die Java Source Dateien enthalten!' if Dir[pathname + '**/*.java'].empty?
        validation_message = 'Test muss die Class Dateien enthalten!' if Dir[pathname + '**/*.class'].empty?

        break unless validation_message.nil?

        compiled, error = compiles?(path)
        validation_message = 'Test compiliert ohne Abgabe. Test enthält vermutlich die zu testende Klasse und verdeckt sie somit. Bitte entfernen sie die zu testende Klasse aus der Jar-Datei.' if compiled

        break unless validation_message.nil?

        unzip stub_file, path

        compiled, error = compiles?(path)
        validation_message = ['Test kann mit dem angegeben Stub nicht compiliert werden.', error] unless compiled
      end

      return validation_message
    end

    def validate_submission_structure(submission_file, test_file)
      validation_message = validate_is_jar? submission_file

      return validation_message unless validation_message.nil?

      run_unzipped submission_file do |path|
        pathname = Pathname.new path

        validation_message = 'Abgabe muss die Java Source Dateien enthalten!' if Dir[pathname + '**/*.java'].empty?
        validation_message = 'Abgabe muss die Class Dateien enthalten!' if Dir[pathname + '**/*.class'].empty?

        break unless validation_message.nil?

        unzip test_file, path # Unzip now because it will break the above checks

        compiled, error = compiles?(path)
        validation_message = ['Abgabe kann nicht compiliert werden.', error] unless compiled
      end

      return validation_message
    end

    def moss_language
      MOSS_LANGUAGE
    end

    # calls block with all source file contents that may be relevant to moss
    def source_files(carrierwave_uploader)
      run_unzipped carrierwave_uploader do |path|
        path_name = Pathname.new path

        source_files = Dir[path_name + '**/*.java']

        source_files.each do |source_file|
          yield File.basename(source_file), File.open(source_file)
        end
      end
    end

    def test(task, submission, test_suit)
      result = nil
      testJars = task.test_jar_list

      run_unzipped submission.submission, *testJars do |working_dir|
        result = run_json_reporter(working_dir,
                                   test_suit,
                                   {jars: testJars << submission.submission.current_path})

      end
      return result
    end

    def java_path
      if (not JavaSubmissionTester.default_java_version.nil?) && JavaSubmissionTester.default_java_version==9
        return JavaSubmissionTester.java_9_path
      else
        return JavaSubmissionTester.java_8_path
      end
    end
  end
end
