require 'spec_helper'

RSpec.describe ::JavaSubmissionTester::JavaHelper do
  include ::JavaSubmissionTester::JavaHelper
  def java_path
    if (not JavaSubmissionTester.default_java_version.nil?) && JavaSubmissionTester.default_java_version==9
      return JavaSubmissionTester.java_9_path
    else
      return JavaSubmissionTester.java_8_path
    end
  end
  current_file = Pathname.new(File.dirname(__FILE__))
  haskellFile = TEST_RSC_DIR.join('io-solution.hs')

  it "should fail with 'not a jar'" do
    expect(validate_is_jar?(haskellFile)).to include("jar sein")
  end

  it "should fail compiling a jar with only class files" do
    run_unzipped TEST_RSC_DIR.join('only_classes.jar') do |path|
      compiled, error = compiles?(path)
      expect(compiled).to be(false)
      expect(error).not_to be_empty
    end
  end
  it "should compile a valid jar" do
    run_unzipped TEST_RSC_DIR.join('compiling_jar.jar') do |path|
      compiled, error = compiles?(path)
      expect(compiled).to be(true)
      expect(error).to be_empty
    end
  end

  it "should setup a bwrap environment" do
    cmd = setup_bubblewrap([current_file],[haskellFile], "ls")
    JavaSubmissionTester.bubblewrap_dependencies.each do |dependency|
      expect(cmd).to include("--ro-bind #{dependency}")
    end
    expect(cmd).to include("--ro-bind #{::JavaSubmissionTester::JavaHelper::LIB_DIR} /tmp/libs")
    expect(cmd).to include("--ro-bind #{java_path()[:jvm_cfg]}")
    expect(cmd).to include("--ro-bind #{current_file}")
    expect(cmd).to include("--bind #{haskellFile}")
    expect(cmd.strip).to end_with("ls")
  end

  it "should setup a java command" do
    opts = {
      properties: "-Doutput=out",
      jars: "/tmp/awesome-jar.jar"
    }
    suite = "awesome-suite"
    outputFile = "awesome-out.txt"
    library_cp = Dir[::JavaSubmissionTester::JavaHelper::LIB_DIR+"*.jar"].map { |p| "libs/#{File.basename(p)}"}
    cmd = java_cmd(outputFile, suite, opts)
    expect(cmd).to include(opts[:properties])
    expect(cmd).to include(":awesome-jar.jar")
    expect(cmd).to include(suite)
    expect(cmd).to include(outputFile)
    library_cp.each do |jar|
      expect(cmd).to include(jar)
    end
  end
end
